/***************************************
 * Name   : Lab 05 Template            *
 * Author : IamN0tRobot                *
 ***************************************/
#include "3DoTConfig.h"

void setup() {
  Serial.begin(9600);
  init3DoT();
  digitalWrite(LED_BUILTIN, HIGH);
  //delay(15000);   // 15 seconds

  // 3.83v from battery sensor     **** NEW ****
  uint16_t batt = analogRead(A5);
  float Vbatt = 2*(2.56/1024)* (float)batt;  // battery voltage divided by 2 
  Serial.print("Battery Voltage = ");   // USB
  Serial.println(Vbatt);
}

void loop() {
  delay(2000);
  uint32_t start_of_test = millis();
  takeAStep();
  turnLeft(390);    // 380 a little short 400 long remove argument to let FSM provide estimate
}

void takeAStep() {
  /* Finite State Machine */
  static int state = 0;    // fsm state
  do {
    /* Read the inside IR sensors */
    static uint8_t sensorLeftInside  = LOW;
    static uint8_t sensorRightInside = LOW;
    sensorLeftInside  = sensorRead(IR_L_I,sensorLeftInside);  // White = 0, LOW
    sensorRightInside = sensorRead(IR_R_I,sensorRightInside); // Black = 1, HIGH
    /* Run the path following algorithm */
    static uint32_t activate_sensors;
    switch(state) {
      case 0:        // start
        activate_sensors = millis() + 250;  // transit across square should take approx. 500 ms
        state = 1;   
        break;
      case 1:        // walk
        walk();      // slows down over walls and hedges
        if(millis() > activate_sensors){

        }
        break;
      /* continue walking until both inside sensors are on the line */
      case 2:        // align robot to the line (pivot ccw)

        break;
      case 3:        // align robot to the line (pivot cw)

        break;
     } // end switch w/o default state
  } while(state !=0);
}

// Timed turn versions
void turnLeft(uint32_t timed_turn) {    // unsigned long
    setMotor(spin_left);
    delay(timed_turn);
    setMotor(brake); 
}

void turnRight(uint32_t timed_turn) {    // unsigned long
    setMotor(spin_right);
    delay(timed_turn);
    setMotor(brake); 
}

/* FSM Version */
bool turnLeft() {
  setMotor(spin_left);
  static int state = 0; 
  static uint8_t sensorRight = BLACK;
  setMotor(spin_left);    // start the turn
  do {
  switch(state){
    case 0:
      state++;     // continue to state 1
      break;
    case 1:
      // loop on this state until right sensor enters the room
      sensorRight = sensorRead(IR_R_O,sensorRight);
      if (sensorRight == WHITE) state++;
      break;
    case 2:
      // continue rotation through the room
      sensorRight = sensorRead(IR_R_O,sensorRight);
      if (sensorRight == BLACK)state++;
      break;
    case 3:
      // rotation is complete when the right sensor enters the room 
      sensorRight = sensorRead(IR_R_O,sensorRight);
      if (sensorRight == WHITE){
        setMotor(brake);
        state = 0;
      }
      break; 
    }  
  } while(state !=0);
  
  return true; 
}

void walk() {
  /* Initialization */
  static uint8_t sensorLeft = LOW;
  static uint8_t sensorRight = LOW;
  /* Read IR sensors */
  sensorLeft  = sensorRead(IR_L_O,sensorLeft);  // White = 0, LOW
  sensorRight = sensorRead(IR_R_O,sensorRight); // Black = 1, HIGH
  // To follow a path the ir sensor controls the opposite motor
  analogWrite(AIN1, 0);     // left motor
  analogWrite(AIN2, touchAWall(sensorRight,motorLeftHIGH));
  analogWrite(BIN1, 0);     // right motor
  analogWrite(BIN2, touchAWall(sensorLeft,motorRightHIGH));
}

uint8_t touchAWall(uint8_t sensor_reading, uint8_t motorHIGH) {   // labs walk
  uint8_t motorSpeed;
  if (sensor_reading == LOW){
    // white, set motor to max value
    motorSpeed = motorHIGH;  
  } else {
    // black, do not risk crossing the line by pivoting in place.   **** NEW ****
    motorSpeed = LOW;  
  }
  return motorSpeed; 
}

uint8_t sensorRead(uint8_t sensor_pin, uint8_t current_level) {
  uint8_t  logic_level = current_level;   // initial value white, motor ON 
  uint16_t analog_value = analogRead(sensor_pin);
  if (analog_value >= trigger1) logic_level = HIGH;
  if (analog_value <= trigger0) logic_level = LOW;
  /* else do not change */
  return logic_level;
}




     
